@@ -11,6 +11,7 @@ import (
 	"time"
 
 	"github.com/goreleaser/goreleaser/internal/artifact"
+	"github.com/goreleaser/goreleaser/internal/testctx"
 	"github.com/goreleaser/goreleaser/internal/testlib"
 	"github.com/goreleaser/goreleaser/internal/tmpl"
 	api "github.com/goreleaser/goreleaser/pkg/build"
@@ -77,6 +78,9 @@ func TestWithDefaults(t *testing.T) {
 				"linux_arm64",
 				"darwin_amd64_v1",
 				"darwin_arm64",
+				"windows_amd64_v1",
+				"windows_arm64",
+				"windows_386",
 			},
 			goBinary: "go",
 		},
@@ -167,6 +171,9 @@ func TestWithDefaults(t *testing.T) {
 				"linux_arm64",
 				"darwin_amd64_v1",
 				"darwin_arm64",
+				"windows_amd64_v1",
+				"windows_arm64",
+				"windows_386",
 			},
 			goBinary: "go",
 		},
@@ -182,6 +189,9 @@ func TestWithDefaults(t *testing.T) {
 				"linux_arm64",
 				"darwin_amd64_v1",
 				"darwin_arm64",
+				"windows_amd64_v1",
+				"windows_arm64",
+				"windows_386",
 			},
 			goBinary: "go",
 		},
@@ -226,13 +236,11 @@ func TestWithDefaults(t *testing.T) {
 			if testcase.build.GoBinary != "" && testcase.build.GoBinary != "go" {
 				createFakeGoBinaryWithVersion(t, testcase.build.GoBinary, "go1.18")
 			}
-			config := config.Project{
+			ctx := testctx.NewWithCfg(config.Project{
 				Builds: []config.Build{
 					testcase.build,
 				},
-			}
-			ctx := context.New(config)
-			ctx.Git.CurrentTag = "5.6.7"
+			}, testctx.WithCurrentTag("5.6.7"))
 			build, err := Default.WithDefaults(ctx.Config.Builds[0])
 			require.NoError(t, err)
 			require.ElementsMatch(t, build.Targets, testcase.targets)
@@ -322,12 +330,11 @@ func TestInvalidTargets(t *testing.T) {
 		},
 	} {
 		t.Run(s, func(t *testing.T) {
-			config := config.Project{
+			ctx := testctx.NewWithCfg(config.Project{
 				Builds: []config.Build{
 					tc.build,
 				},
-			}
-			ctx := context.New(config)
+			})
 			_, err := Default.WithDefaults(ctx.Config.Builds[0])
 			require.EqualError(t, err, tc.expectedErr)
 		})
@@ -337,7 +344,8 @@ func TestInvalidTargets(t *testing.T) {
 func TestBuild(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
+		Env: []string{"GO_FLAGS=-v"},
 		Builds: []config.Build{
 			{
 				ID:     "foo",
@@ -354,7 +362,16 @@ func TestBuild(t *testing.T) {
 				GoBinary: "go",
 				Command:  "build",
 				BuildDetails: config.BuildDetails{
-					Env:      []string{"GO111MODULE=off"},
+					Env: []string{
+						"GO111MODULE=off",
+						`TEST_T={{- if eq .Os "windows" -}}
+						w
+						{{- else if eq .Os "darwin" -}}
+						d
+						{{- else if eq .Os "linux" -}}
+						l
+						{{- end -}}`,
+					},
 					Asmflags: []string{".=", "all="},
 					Gcflags:  []string{"all="},
 					Flags:    []string{"{{.Env.GO_FLAGS}}"},
@@ -362,11 +379,7 @@ func TestBuild(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Env["GO_FLAGS"] = "-v"
-	ctx.Git.CurrentTag = "v5.6.7"
-	ctx.Version = ctx.Git.CurrentTag
+	}, testctx.WithCurrentTag("v5.6.7"), testctx.WithVersion("v5.6.7"))
 	build := ctx.Config.Builds[0]
 	for _, target := range build.Targets {
 		var ext string
@@ -417,6 +430,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    "",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T=l"},
 			},
 		},
 		{
@@ -430,6 +444,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    "",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T=l"},
 			},
 		},
 		{
@@ -443,6 +458,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    "",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T=l"},
 			},
 		},
 		{
@@ -455,6 +471,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    "",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T=d"},
 			},
 		},
 		{
@@ -468,6 +485,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    "",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T=l"},
 			},
 		},
 		{
@@ -480,6 +498,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    ".exe",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T=w"},
 			},
 		},
 		{
@@ -492,6 +511,7 @@ func TestBuild(t *testing.T) {
 				artifact.ExtraExt:    ".wasm",
 				artifact.ExtraBinary: "foo-v5.6.7",
 				artifact.ExtraID:     "foo",
+				"testEnvs":           []string{"TEST_T="},
 			},
 		},
 	})
@@ -515,13 +535,42 @@ func TestBuild(t *testing.T) {
 	}
 }
 
+func TestBuildInvalidEnv(t *testing.T) {
+	folder := testlib.Mktmp(t)
+	writeGoodMain(t, folder)
+	ctx := testctx.NewWithCfg(config.Project{
+		Builds: []config.Build{
+			{
+				ID:     "foo",
+				Dir:    ".",
+				Binary: "foo",
+				Targets: []string{
+					runtimeTarget,
+				},
+				GoBinary: "go",
+				BuildDetails: config.BuildDetails{
+					Env: []string{"GO111MODULE={{ .Nope }}"},
+				},
+			},
+		},
+	}, testctx.WithCurrentTag("5.6.7"))
+	build := ctx.Config.Builds[0]
+	err := Default.Build(ctx, build, api.Options{
+		Target: runtimeTarget,
+		Name:   build.Binary,
+		Path:   filepath.Join(folder, "dist", runtimeTarget, build.Binary),
+		Ext:    "",
+	})
+	testlib.RequireTemplateError(t, err)
+}
+
 func TestBuildCodeInSubdir(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	subdir := filepath.Join(folder, "bar")
 	err := os.Mkdir(subdir, 0o755)
 	require.NoError(t, err)
 	writeGoodMain(t, subdir)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				ID:     "foo",
@@ -537,9 +586,7 @@ func TestBuildCodeInSubdir(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	build := ctx.Config.Builds[0]
 	err = Default.Build(ctx, build, api.Options{
 		Target: runtimeTarget,
@@ -554,7 +601,7 @@ func TestBuildWithDotGoDir(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	require.NoError(t, os.Mkdir(filepath.Join(folder, ".go"), 0o755))
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				ID:       "foo",
@@ -567,9 +614,7 @@ func TestBuildWithDotGoDir(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	build := ctx.Config.Builds[0]
 	require.NoError(t, Default.Build(ctx, build, api.Options{
 		Target: runtimeTarget,
@@ -582,7 +627,7 @@ func TestBuildWithDotGoDir(t *testing.T) {
 func TestBuildFailed(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				ID: "buildid",
@@ -596,9 +641,7 @@ func TestBuildFailed(t *testing.T) {
 				Command:  "build",
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	err := Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 		Target: "darwin_amd64",
 	})
@@ -609,7 +652,7 @@ func TestBuildFailed(t *testing.T) {
 func TestRunInvalidAsmflags(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				Binary: "nametest",
@@ -621,9 +664,7 @@ func TestRunInvalidAsmflags(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	err := Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 		Target: runtimeTarget,
 	})
@@ -633,7 +674,7 @@ func TestRunInvalidAsmflags(t *testing.T) {
 func TestRunInvalidGcflags(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				Binary: "nametest",
@@ -645,9 +686,7 @@ func TestRunInvalidGcflags(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	err := Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 		Target: runtimeTarget,
 	})
@@ -657,7 +696,7 @@ func TestRunInvalidGcflags(t *testing.T) {
 func TestRunInvalidLdflags(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				Binary: "nametest",
@@ -670,9 +709,7 @@ func TestRunInvalidLdflags(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	err := Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 		Target: runtimeTarget,
 	})
@@ -682,7 +719,7 @@ func TestRunInvalidLdflags(t *testing.T) {
 func TestRunInvalidFlags(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				Binary: "nametest",
@@ -694,8 +731,7 @@ func TestRunInvalidFlags(t *testing.T) {
 				},
 			},
 		},
-	}
-	ctx := context.New(config)
+	})
 	err := Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 		Target: runtimeTarget,
 	})
@@ -707,11 +743,9 @@ func TestRunPipeWithoutMainFunc(t *testing.T) {
 		t.Helper()
 		folder := testlib.Mktmp(t)
 		writeMainWithoutMainFunc(t, folder)
-		config := config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			Builds: []config.Build{{Binary: "no-main"}},
-		}
-		ctx := context.New(config)
-		ctx.Git.CurrentTag = "5.6.7"
+		}, testctx.WithCurrentTag("5.6.7"))
 		return ctx
 	}
 	t.Run("empty", func(t *testing.T) {
@@ -724,9 +758,9 @@ func TestRunPipeWithoutMainFunc(t *testing.T) {
 	t.Run("not main.go", func(t *testing.T) {
 		ctx := newCtx(t)
 		ctx.Config.Builds[0].Main = "foo.go"
-		require.EqualError(t, Default.Build(ctx, ctx.Config.Builds[0], api.Options{
+		require.ErrorIs(t, Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 			Target: runtimeTarget,
-		}), `couldn't find main file: stat foo.go: no such file or directory`)
+		}), os.ErrNotExist)
 	})
 	t.Run("glob", func(t *testing.T) {
 		ctx := newCtx(t)
@@ -758,19 +792,17 @@ func TestRunPipeWithoutMainFunc(t *testing.T) {
 func TestBuildTests(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeTest(t, folder)
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{{
 			Binary:  "foo.test",
 			Command: "test",
 			BuildDetails: config.BuildDetails{
 				Flags: []string{"-c"},
 			},
+			NoMainCheck: true,
 		}},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
-	ctx.Config.Builds[0].NoMainCheck = true
-	build, err := Default.WithDefaults(config.Builds[0])
+	}, testctx.WithCurrentTag("5.6.7"))
+	build, err := Default.WithDefaults(ctx.Config.Builds[0])
 	require.NoError(t, err)
 	require.NoError(t, Default.Build(ctx, build, api.Options{
 		Target: runtimeTarget,
@@ -803,7 +835,7 @@ import _ "github.com/goreleaser/goreleaser"
 	cmd.Dir = proxied
 	require.NoError(t, cmd.Run())
 
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		GoMod: config.GoMod{
 			Proxy: true,
 		},
@@ -821,8 +853,7 @@ import _ "github.com/goreleaser/goreleaser"
 				Command:  "build",
 			},
 		},
-	}
-	ctx := context.New(config)
+	})
 
 	require.NoError(t, Default.Build(ctx, ctx.Config.Builds[0], api.Options{
 		Target: runtimeTarget,
@@ -836,7 +867,7 @@ func TestRunPipeWithMainFuncNotInMainGoFile(t *testing.T) {
 		[]byte("package main\nfunc main() {println(0)}"),
 		0o644,
 	))
-	config := config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Builds: []config.Build{
 			{
 				Binary: "foo",
@@ -851,9 +882,7 @@ func TestRunPipeWithMainFuncNotInMainGoFile(t *testing.T) {
 				Command:  "build",
 			},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Git.CurrentTag = "5.6.7"
+	}, testctx.WithCurrentTag("5.6.7"))
 	t.Run("empty", func(t *testing.T) {
 		ctx.Config.Builds[0].Main = ""
 		require.NoError(t, Default.Build(ctx, ctx.Config.Builds[0], api.Options{
@@ -877,19 +906,16 @@ func TestRunPipeWithMainFuncNotInMainGoFile(t *testing.T) {
 func TestLdFlagsFullTemplate(t *testing.T) {
 	run := time.Now().UTC()
 	commit := time.Now().AddDate(-1, 0, 0)
-
-	ctx := &context.Context{
-		Git: context.GitInfo{
-			CurrentTag: "v1.2.3",
-			Commit:     "123",
-			CommitDate: commit,
-		},
-		Date:    run,
-		Version: "1.2.3",
-		Env:     map[string]string{"FOO": "123"},
-	}
+	ctx := testctx.New(
+		testctx.WithCurrentTag("v1.2.3"),
+		testctx.WithCommit("123"),
+		testctx.WithCommitDate(commit),
+		testctx.WithVersion("1.2.3"),
+		testctx.WithEnv(map[string]string{"FOO": "123"}),
+		testctx.WithDate(run),
+	)
 	artifact := &artifact.Artifact{Goarch: "amd64"}
-	flags, err := tmpl.New(ctx).WithArtifact(artifact, map[string]string{}).
+	flags, err := tmpl.New(ctx).WithArtifact(artifact).
 		Apply(`-s -w -X main.version={{.Version}} -X main.tag={{.Tag}} -X main.date={{.Date}} -X main.commit={{.Commit}} -X "main.foo={{.Env.FOO}}" -X main.time={{ time "20060102" }} -X main.arch={{.Arch}} -X main.commitDate={{.CommitDate}}`)
 	require.NoError(t, err)
 	require.Contains(t, flags, "-s -w")
@@ -909,8 +935,7 @@ func TestInvalidTemplate(t *testing.T) {
 		"{{.Env.NOPE}}",
 	} {
 		t.Run(template, func(t *testing.T) {
-			ctx := context.New(config.Project{})
-			ctx.Git.CurrentTag = "3.4.1"
+			ctx := testctx.New(testctx.WithCurrentTag("3.4.1"))
 			flags, err := tmpl.New(ctx).Apply(template)
 			testlib.RequireTemplateError(t, err)
 			require.Empty(t, flags)
@@ -919,10 +944,10 @@ func TestInvalidTemplate(t *testing.T) {
 }
 
 func TestProcessFlags(t *testing.T) {
-	ctx := &context.Context{
-		Version: "1.2.3",
-	}
-	ctx.Git.CurrentTag = "5.6.7"
+	ctx := testctx.New(
+		testctx.WithVersion("1.2.3"),
+		testctx.WithCurrentTag("5.6.7"),
+	)
 
 	artifact := &artifact.Artifact{
 		Name:   "name",
@@ -961,12 +986,10 @@ func TestProcessFlags(t *testing.T) {
 }
 
 func TestProcessFlagsInvalid(t *testing.T) {
-	ctx := &context.Context{}
-
+	ctx := testctx.New()
 	source := []string{
 		"{{.Version}",
 	}
-
 	flags, err := processFlags(ctx, &artifact.Artifact{}, []string{}, source, "-testflag=")
 	testlib.RequireTemplateError(t, err)
 	require.Nil(t, flags)
@@ -979,9 +1002,10 @@ func TestBuildModTimestamp(t *testing.T) {
 	folder := testlib.Mktmp(t)
 	writeGoodMain(t, folder)
 
-	config := config.Project{
-		Builds: []config.Build{
-			{
+	ctx := testctx.NewWithCfg(
+		config.Project{
+			Env: []string{"GO_FLAGS=-v"},
+			Builds: []config.Build{{
 				ID:     "foo",
 				Binary: "bin/foo-{{ .Version }}",
 				Targets: []string{
@@ -1002,13 +1026,11 @@ func TestBuildModTimestamp(t *testing.T) {
 				ModTimestamp: fmt.Sprintf("%d", modTime.Unix()),
 				GoBinary:     "go",
 				Command:      "build",
-			},
+			}},
 		},
-	}
-	ctx := context.New(config)
-	ctx.Env["GO_FLAGS"] = "-v"
-	ctx.Git.CurrentTag = "v5.6.7"
-	ctx.Version = ctx.Git.CurrentTag
+		testctx.WithCurrentTag("v5.6.7"),
+		testctx.WithVersion("5.6.7"),
+	)
 	build := ctx.Config.Builds[0]
 	for _, target := range build.Targets {
 		var ext string
@@ -1046,15 +1068,15 @@ func TestBuildModTimestamp(t *testing.T) {
 func TestBuildGoBuildLine(t *testing.T) {
 	requireEqualCmd := func(tb testing.TB, build config.Build, expected []string) {
 		tb.Helper()
-		cfg := config.Project{
-			Builds: []config.Build{build},
-		}
-		ctx := context.New(cfg)
-		ctx.Version = "1.2.3"
-		ctx.Git.Commit = "aaa"
-
+		ctx := testctx.NewWithCfg(
+			config.Project{
+				Builds: []config.Build{build},
+			},
+			testctx.WithVersion("1.2.3"),
+			testctx.WithGitInfo(context.GitInfo{Commit: "aaa"}),
+		)
 		options := api.Options{
-			Path:   cfg.Builds[0].Binary,
+			Path:   ctx.Config.Builds[0].Binary,
 			Goos:   "linux",
 			Goarch: "amd64",
 		}
@@ -1188,7 +1210,7 @@ func TestBuildGoBuildLine(t *testing.T) {
 func TestOverrides(t *testing.T) {
 	t.Run("linux amd64", func(t *testing.T) {
 		dets, err := withOverrides(
-			context.New(config.Project{}),
+			testctx.New(),
 			config.Build{
 				BuildDetails: config.BuildDetails{
 					Ldflags: []string{"original"},
@@ -1216,7 +1238,7 @@ func TestOverrides(t *testing.T) {
 
 	t.Run("single sided", func(t *testing.T) {
 		dets, err := withOverrides(
-			context.New(config.Project{}),
+			testctx.New(),
 			config.Build{
 				BuildDetails: config.BuildDetails{},
 				BuildDetailsOverrides: []config.BuildDetailsOverride{
@@ -1248,7 +1270,7 @@ func TestOverrides(t *testing.T) {
 
 	t.Run("with template", func(t *testing.T) {
 		dets, err := withOverrides(
-			context.New(config.Project{}),
+			testctx.New(),
 			config.Build{
 				BuildDetails: config.BuildDetails{
 					Ldflags:  []string{"original"},
@@ -1278,7 +1300,7 @@ func TestOverrides(t *testing.T) {
 
 	t.Run("with invalid template", func(t *testing.T) {
 		_, err := withOverrides(
-			context.New(config.Project{}),
+			testctx.New(),
 			config.Build{
 				BuildDetailsOverrides: []config.BuildDetailsOverride{
 					{
@@ -1295,7 +1317,7 @@ func TestOverrides(t *testing.T) {
 
 	t.Run("with goarm", func(t *testing.T) {
 		dets, err := withOverrides(
-			context.New(config.Project{}),
+			testctx.New(),
 			config.Build{
 				BuildDetails: config.BuildDetails{
 					Ldflags: []string{"original"},
@@ -1325,7 +1347,7 @@ func TestOverrides(t *testing.T) {
 
 	t.Run("with gomips", func(t *testing.T) {
 		dets, err := withOverrides(
-			context.New(config.Project{}),
+			testctx.New(),
 			config.Build{
 				BuildDetails: config.BuildDetails{
 					Ldflags: []string{"original"},
