@@ -10,15 +10,15 @@ import (
 	"text/template"
 
 	"github.com/goreleaser/goreleaser/internal/artifact"
+	"github.com/goreleaser/goreleaser/internal/testctx"
 	"github.com/goreleaser/goreleaser/pkg/config"
-	"github.com/goreleaser/goreleaser/pkg/context"
 	"github.com/stretchr/testify/require"
 )
 
 func TestNewGitHubClient(t *testing.T) {
 	t.Run("good urls", func(t *testing.T) {
 		githubURL := "https://github.mycompany.com"
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			GitHubURLs: config.GitHubURLs{
 				API:    githubURL + "/api",
 				Upload: githubURL + "/upload",
@@ -35,7 +35,7 @@ func TestNewGitHubClient(t *testing.T) {
 	})
 
 	t.Run("bad api url", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			GitHubURLs: config.GitHubURLs{
 				API:    "://github.mycompany.com/api",
 				Upload: "https://github.mycompany.com/upload",
@@ -47,7 +47,7 @@ func TestNewGitHubClient(t *testing.T) {
 	})
 
 	t.Run("bad upload url", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			GitHubURLs: config.GitHubURLs{
 				API:    "https://github.mycompany.com/api",
 				Upload: "not a url:4994",
@@ -60,7 +60,7 @@ func TestNewGitHubClient(t *testing.T) {
 
 	t.Run("template", func(t *testing.T) {
 		githubURL := "https://github.mycompany.com"
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			Env: []string{
 				fmt.Sprintf("GORELEASER_TEST_GITHUB_URLS_API=%s/api", githubURL),
 				fmt.Sprintf("GORELEASER_TEST_GITHUB_URLS_UPLOAD=%s/upload", githubURL),
@@ -81,7 +81,7 @@ func TestNewGitHubClient(t *testing.T) {
 	})
 
 	t.Run("template invalid api", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			GitHubURLs: config.GitHubURLs{
 				API: "{{ .Env.GORELEASER_NOT_EXISTS }}",
 			},
@@ -92,7 +92,7 @@ func TestNewGitHubClient(t *testing.T) {
 	})
 
 	t.Run("template invalid upload", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			GitHubURLs: config.GitHubURLs{
 				API:    "https://github.mycompany.com/api",
 				Upload: "{{ .Env.GORELEASER_NOT_EXISTS }}",
@@ -104,7 +104,7 @@ func TestNewGitHubClient(t *testing.T) {
 	})
 
 	t.Run("template invalid", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			GitHubURLs: config.GitHubURLs{
 				API: "{{.dddddddddd",
 			},
@@ -116,7 +116,7 @@ func TestNewGitHubClient(t *testing.T) {
 }
 
 func TestGitHubUploadReleaseIDNotInt(t *testing.T) {
-	ctx := context.New(config.Project{})
+	ctx := testctx.New()
 	client, err := NewGitHub(ctx, ctx.Token)
 	require.NoError(t, err)
 
@@ -158,7 +158,7 @@ func TestGitHubReleaseURLTemplate(t *testing.T) {
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			ctx := context.New(config.Project{
+			ctx := testctx.NewWithCfg(config.Project{
 				Env: []string{
 					"GORELEASER_TEST_GITHUB_URLS_DOWNLOAD=https://github.mycompany.com",
 				},
@@ -188,7 +188,7 @@ func TestGitHubReleaseURLTemplate(t *testing.T) {
 }
 
 func TestGitHubCreateReleaseWrongNameTemplate(t *testing.T) {
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Release: config.Release{
 			NameTemplate: "{{.dddddddddd",
 		},
@@ -213,7 +213,7 @@ func TestGithubGetDefaultBranch(t *testing.T) {
 	}))
 	defer srv.Close()
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		GitHubURLs: config.GitHubURLs{
 			API: srv.URL + "/",
 		},
@@ -243,7 +243,7 @@ func TestGithubGetDefaultBranchErr(t *testing.T) {
 	}))
 	defer srv.Close()
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		GitHubURLs: config.GitHubURLs{
 			API: srv.URL + "/",
 		},
@@ -274,7 +274,7 @@ func TestChangelog(t *testing.T) {
 	}))
 	defer srv.Close()
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		GitHubURLs: config.GitHubURLs{
 			API: srv.URL + "/",
 		},
@@ -289,7 +289,7 @@ func TestChangelog(t *testing.T) {
 
 	log, err := client.Changelog(ctx, repo, "v1.0.0", "v1.1.0")
 	require.NoError(t, err)
-	require.Equal(t, "6dcb09b: Fix all the bugs (@octocat)", log)
+	require.Equal(t, "6dcb09b5b57875f334f61aebed695e2e4193db5e: Fix all the bugs (@octocat)", log)
 }
 
 func TestReleaseNotes(t *testing.T) {
@@ -306,7 +306,7 @@ func TestReleaseNotes(t *testing.T) {
 	}))
 	defer srv.Close()
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		GitHubURLs: config.GitHubURLs{
 			API: srv.URL + "/",
 		},
@@ -323,3 +323,33 @@ func TestReleaseNotes(t *testing.T) {
 	require.NoError(t, err)
 	require.Equal(t, "**Full Changelog**: https://github.com/someone/something/compare/v1.0.0...v1.1.0", log)
 }
+
+func TestCloseMilestone(t *testing.T) {
+	srv := httptest.NewServer(http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
+		defer r.Body.Close()
+		t.Log(r.URL.Path)
+
+		if r.URL.Path == "/repos/someone/something/milestones" {
+			r, err := os.Open("testdata/github/milestones.json")
+			require.NoError(t, err)
+			_, err = io.Copy(w, r)
+			require.NoError(t, err)
+			return
+		}
+	}))
+	defer srv.Close()
+
+	ctx := testctx.NewWithCfg(config.Project{
+		GitHubURLs: config.GitHubURLs{
+			API: srv.URL + "/",
+		},
+	})
+	client, err := NewGitHub(ctx, "test-token")
+	require.NoError(t, err)
+	repo := Repo{
+		Owner: "someone",
+		Name:  "something",
+	}
+
+	require.NoError(t, client.CloseMilestone(ctx, repo, "v1.13.0"))
+}
