@@ -31,6 +31,18 @@ type Pipe struct{}
 func (Pipe) String() string                 { return "docker images" }
 func (Pipe) Skip(ctx *context.Context) bool { return len(ctx.Config.Dockers) == 0 || ctx.SkipDocker }
 
+func (Pipe) Dependencies(ctx *context.Context) []string {
+	var cmds []string
+	for _, s := range ctx.Config.Dockers {
+		switch s.Use {
+		case useDocker, useBuildx:
+			cmds = append(cmds, "docker")
+			// TODO: how to check if buildx is installed
+		}
+	}
+	return cmds
+}
+
 // Default sets the pipe defaults.
 func (Pipe) Default(ctx *context.Context) error {
 	ids := ids.New("dockers")
@@ -61,11 +73,6 @@ func (Pipe) Default(ctx *context.Context) error {
 		if err := validateImager(docker.Use); err != nil {
 			return err
 		}
-		for _, f := range docker.Files {
-			if f == "." || strings.HasPrefix(f, ctx.Config.Dist) {
-				return fmt.Errorf("invalid docker.files: can't be . or inside dist folder: %s", f)
-			}
-		}
 	}
 	return ids.Validate()
 }
@@ -129,9 +136,6 @@ func (Pipe) Run(ctx *context.Context) error {
 			}
 			artifacts := ctx.Artifacts.Filter(artifact.And(filters...))
 			log.WithField("artifacts", artifacts.Paths()).Debug("found artifacts")
-			if len(artifacts.Paths()) == 0 {
-				log.Warn("not binaries or packages found for the given platform - COPY/ADD may not work")
-			}
 			return process(ctx, docker, artifacts.List())
 		})
 	}
@@ -145,7 +149,10 @@ func (Pipe) Run(ctx *context.Context) error {
 }
 
 func process(ctx *context.Context, docker config.Docker, artifacts []*artifact.Artifact) error {
-	tmp, err := os.MkdirTemp(ctx.Config.Dist, "goreleaserdocker")
+	if len(artifacts) == 0 {
+		log.Warn("not binaries or packages found for the given platform - COPY/ADD may not work")
+	}
+	tmp, err := os.MkdirTemp("", "goreleaserdocker")
 	if err != nil {
 		return fmt.Errorf("failed to create temporary dir: %w", err)
 	}
