@@ -62,6 +62,11 @@ func (Pipe) Default(ctx *context.Context) error {
 		if len(fpm.Replacements) != 0 {
 			deprecate.Notice(ctx, "nfpms.replacements")
 		}
+		for _, rfpm := range fpm.Overrides {
+			if len(rfpm.Replacements) != 0 {
+				deprecate.Notice(ctx, "nfpms.replacements")
+			}
+		}
 		ids.Inc(fpm.ID)
 	}
 
@@ -145,7 +150,11 @@ func create(ctx *context.Context, fpm config.NFPM, format string, binaries []*ar
 	arch := infoArch + binaries[0].Goamd64                                  // unique arch key
 	infoPlatform := binaries[0].Goos
 	if infoPlatform == "ios" {
-		infoPlatform = "iphoneos-arm64"
+		if format == "deb" {
+			infoPlatform = "iphoneos-arm64"
+		} else {
+			return nil
+		}
 	}
 
 	bindDir := fpm.Bindir
@@ -213,6 +222,11 @@ func create(ctx *context.Context, fpm config.NFPM, format string, binaries []*ar
 		return err
 	}
 
+	apkKeyName, err := t.Apply(overridden.APK.Signature.KeyName)
+	if err != nil {
+		return err
+	}
+
 	contents := files.Contents{}
 	for _, content := range overridden.Contents {
 		src, err := t.Apply(content.Source)
@@ -353,7 +367,7 @@ func create(ctx *context.Context, fpm config.NFPM, format string, binaries []*ar
 						KeyFile:       apkKeyFile,
 						KeyPassphrase: getPassphraseFromEnv(ctx, "APK", fpm.ID),
 					},
-					KeyName: overridden.APK.Signature.KeyName,
+					KeyName: apkKeyName,
 				},
 				Scripts: nfpm.APKScripts{
 					PreUpgrade:  overridden.APK.Scripts.PreUpgrade,
@@ -382,14 +396,6 @@ func create(ctx *context.Context, fpm config.NFPM, format string, binaries []*ar
 		return err
 	}
 
-	info = nfpm.WithDefaults(info)
-	name, err := t.WithExtraFields(tmpl.Fields{
-		"ConventionalFileName": packager.ConventionalFileName(info),
-	}).Apply(overridden.FileNameTemplate)
-	if err != nil {
-		return err
-	}
-
 	ext := "." + format
 	if packager, ok := packager.(nfpm.PackagerWithExtension); ok {
 		if format != "termux.deb" {
@@ -397,6 +403,15 @@ func create(ctx *context.Context, fpm config.NFPM, format string, binaries []*ar
 		}
 	}
 
+	info = nfpm.WithDefaults(info)
+	name, err := t.WithExtraFields(tmpl.Fields{
+		"ConventionalFileName":  packager.ConventionalFileName(info),
+		"ConventionalExtension": ext,
+	}).Apply(overridden.FileNameTemplate)
+	if err != nil {
+		return err
+	}
+
 	if !strings.HasSuffix(name, ext) {
 		name = name + ext
 	}
