@@ -4,9 +4,10 @@ import (
 	"crypto/tls"
 	"fmt"
 
-	"github.com/caarlos0/env/v6"
+	"github.com/caarlos0/env/v7"
 	"github.com/caarlos0/log"
 	"github.com/goreleaser/goreleaser/internal/tmpl"
+	"github.com/goreleaser/goreleaser/pkg/config"
 	"github.com/goreleaser/goreleaser/pkg/context"
 	gomail "gopkg.in/mail.v2"
 )
@@ -22,9 +23,9 @@ func (Pipe) String() string                 { return "smtp" }
 func (Pipe) Skip(ctx *context.Context) bool { return !ctx.Config.Announce.SMTP.Enabled }
 
 type Config struct {
-	Host     string `env:"SMTP_HOST,notEmpty"`
-	Port     int    `env:"SMTP_PORT,notEmpty"`
-	Username string `env:"SMTP_USERNAME,notEmpty"`
+	Host     string `env:"SMTP_HOST"`
+	Port     int    `env:"SMTP_PORT"`
+	Username string `env:"SMTP_USERNAME"`
 	Password string `env:"SMTP_PASSWORD,notEmpty"`
 }
 
@@ -66,9 +67,9 @@ func (Pipe) Announce(ctx *context.Context) error {
 	// Set E-Mail body. You can set plain text or html with text/html
 	m.SetBody("text/plain", body)
 
-	var cfg Config
-	if err := env.Parse(&cfg); err != nil {
-		return fmt.Errorf("SMTP: %w", err)
+	cfg, err := getConfig(ctx.Config.Announce.SMTP)
+	if err != nil {
+		return err
 	}
 
 	// Settings for SMTP server
@@ -87,3 +88,30 @@ func (Pipe) Announce(ctx *context.Context) error {
 
 	return nil
 }
+
+var (
+	errNoPort     = fmt.Errorf("SMTP: missing smtp.port or $SMTP_PORT")
+	errNoUsername = fmt.Errorf("SMTP: missing smtp.username or $SMTP_USERNAME")
+	errNoHost     = fmt.Errorf("SMTP: missing smtp.host or $SMTP_HOST")
+)
+
+func getConfig(smtp config.SMTP) (Config, error) {
+	cfg := Config{
+		Host:     smtp.Host,
+		Port:     smtp.Port,
+		Username: smtp.Username,
+	}
+	if err := env.Parse(&cfg); err != nil {
+		return cfg, fmt.Errorf("SMTP: %w", err)
+	}
+	if cfg.Username == "" {
+		return cfg, errNoUsername
+	}
+	if cfg.Host == "" {
+		return cfg, errNoHost
+	}
+	if cfg.Port == 0 {
+		return cfg, errNoPort
+	}
+	return cfg, nil
+}
