@@ -99,8 +99,9 @@ const defaultNameTemplate = `{{ .ProjectName }}_{{ .Version }}_{{ .Os }}_{{ .Arc
 // Pipe for snapcraft packaging.
 type Pipe struct{}
 
-func (Pipe) String() string                 { return "snapcraft packages" }
-func (Pipe) Skip(ctx *context.Context) bool { return len(ctx.Config.Snapcrafts) == 0 }
+func (Pipe) String() string                             { return "snapcraft packages" }
+func (Pipe) Skip(ctx *context.Context) bool             { return len(ctx.Config.Snapcrafts) == 0 }
+func (Pipe) Dependencies(ctx *context.Context) []string { return []string{"snapcraft"} }
 
 // Default sets the pipe defaults.
 func (Pipe) Default(ctx *context.Context) error {
@@ -110,6 +111,14 @@ func (Pipe) Default(ctx *context.Context) error {
 		if snap.NameTemplate == "" {
 			snap.NameTemplate = defaultNameTemplate
 		}
+		grade, err := tmpl.New(ctx).Apply(snap.Grade)
+		if err != nil {
+			return err
+		}
+		snap.Grade = grade
+		if snap.Grade == "" {
+			snap.Grade = "stable"
+		}
 		if len(snap.ChannelTemplates) == 0 {
 			switch snap.Grade {
 			case "devel":
