@@ -9,6 +9,7 @@ import (
 	"github.com/goreleaser/goreleaser/internal/artifact"
 	"github.com/goreleaser/goreleaser/internal/gio"
 	"github.com/goreleaser/goreleaser/internal/pipe"
+	"github.com/goreleaser/goreleaser/internal/testctx"
 	"github.com/goreleaser/goreleaser/internal/testlib"
 	"github.com/goreleaser/goreleaser/internal/yaml"
 	"github.com/goreleaser/goreleaser/pkg/config"
@@ -31,13 +32,11 @@ func TestRunPipeMissingInfo(t *testing.T) {
 		pipe.Skip("no summary nor description were provided"): {},
 	} {
 		t.Run(fmt.Sprintf("testing if %v happens", eerr), func(t *testing.T) {
-			ctx := &context.Context{
-				Config: config.Project{
-					Snapcrafts: []config.Snapcraft{
-						snap,
-					},
+			ctx := testctx.NewWithCfg(config.Project{
+				Snapcrafts: []config.Snapcraft{
+					snap,
 				},
-			}
+			})
 			require.Equal(t, eerr, Pipe{}.Run(ctx))
 		})
 	}
@@ -48,7 +47,7 @@ func TestRunPipe(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -77,9 +76,7 @@ func TestRunPipe(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", filepath.Join(dist, "foo"))
 	addBinaries(t, ctx, "bar", filepath.Join(dist, "bar"))
 	require.NoError(t, Pipe{}.Run(ctx))
@@ -92,7 +89,7 @@ func TestBadTemolate(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -103,9 +100,7 @@ func TestBadTemolate(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", filepath.Join(dist, "foo"))
 
 	t.Run("description", func(t *testing.T) {
@@ -126,7 +121,7 @@ func TestRunPipeInvalidNameTemplate(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "foo",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -138,9 +133,7 @@ func TestRunPipeInvalidNameTemplate(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	testlib.RequireTemplateError(t, Pipe{}.Run(ctx))
 }
@@ -150,7 +143,7 @@ func TestRunPipeWithName(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "testprojectname",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -165,9 +158,7 @@ func TestRunPipeWithName(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	require.NoError(t, Pipe{}.Run(ctx))
 	yamlFile, err := os.ReadFile(filepath.Join(dist, "foo_amd64", "prime", "meta", "snap.yaml"))
@@ -186,7 +177,7 @@ func TestRunPipeMetadata(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "testprojectname",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -259,9 +250,7 @@ func TestRunPipeMetadata(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	require.NoError(t, Pipe{}.Run(ctx))
 	yamlFile, err := os.ReadFile(filepath.Join(dist, "foo_amd64", "prime", "meta", "snap.yaml"))
@@ -331,7 +320,7 @@ func TestNoSnapcraftInPath(t *testing.T) {
 		require.NoError(t, os.Setenv("PATH", path))
 	}()
 	require.NoError(t, os.Setenv("PATH", ""))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Snapcrafts: []config.Snapcraft{
 			{
 				Summary:     "dummy",
@@ -347,7 +336,7 @@ func TestRunNoArguments(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "testprojectname",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -365,9 +354,7 @@ func TestRunNoArguments(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	require.NoError(t, Pipe{}.Run(ctx))
 	yamlFile, err := os.ReadFile(filepath.Join(dist, "foo_amd64", "prime", "meta", "snap.yaml"))
@@ -383,7 +370,7 @@ func TestCompleter(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "testprojectname",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -402,9 +389,7 @@ func TestCompleter(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	addBinaries(t, ctx, "bar", dist)
 	require.NoError(t, Pipe{}.Run(ctx))
@@ -422,7 +407,7 @@ func TestCommand(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "testprojectname",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -441,9 +426,7 @@ func TestCommand(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	require.NoError(t, Pipe{}.Run(ctx))
 	yamlFile, err := os.ReadFile(filepath.Join(dist, "foo_amd64", "prime", "meta", "snap.yaml"))
@@ -459,7 +442,7 @@ func TestExtraFile(t *testing.T) {
 	folder := t.TempDir()
 	dist := filepath.Join(folder, "dist")
 	require.NoError(t, os.Mkdir(dist, 0o755))
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "testprojectname",
 		Dist:        dist,
 		Snapcrafts: []config.Snapcraft{
@@ -481,9 +464,7 @@ func TestExtraFile(t *testing.T) {
 				ChannelTemplates: []string{"stable"},
 			},
 		},
-	})
-	ctx.Git.CurrentTag = "v1.2.3"
-	ctx.Version = "1.2.3"
+	}, testctx.WithCurrentTag("v1.2.3"), testctx.WithVersion("1.2.3"))
 	addBinaries(t, ctx, "foo", dist)
 	require.NoError(t, Pipe{}.Run(ctx))
 
@@ -494,23 +475,40 @@ func TestExtraFile(t *testing.T) {
 }
 
 func TestDefault(t *testing.T) {
-	ctx := context.New(config.Project{
-		Builds: []config.Build{
-			{
-				ID: "foo",
-			},
-		},
-		Snapcrafts: []config.Snapcraft{
-			{},
-		},
+	ctx := testctx.NewWithCfg(config.Project{
+		Builds:     []config.Build{{ID: "foo"}},
+		Snapcrafts: []config.Snapcraft{{}},
 	})
 	require.NoError(t, Pipe{}.Default(ctx))
 	require.Equal(t, defaultNameTemplate, ctx.Config.Snapcrafts[0].NameTemplate)
 	require.Equal(t, []string{"foo"}, ctx.Config.Snapcrafts[0].Builds)
+	require.Equal(t, []string{"edge", "beta", "candidate", "stable"}, ctx.Config.Snapcrafts[0].ChannelTemplates)
+	require.Equal(t, "stable", ctx.Config.Snapcrafts[0].Grade)
+}
+
+func TestDefaultGradeTmpl(t *testing.T) {
+	ctx := testctx.NewWithCfg(config.Project{
+		Env:        []string{"Grade=devel"},
+		Builds:     []config.Build{{ID: "foo"}},
+		Snapcrafts: []config.Snapcraft{{Grade: "{{.Env.Grade}}"}},
+	})
+	require.NoError(t, Pipe{}.Default(ctx))
+	require.Equal(t, defaultNameTemplate, ctx.Config.Snapcrafts[0].NameTemplate)
+	require.Equal(t, []string{"foo"}, ctx.Config.Snapcrafts[0].Builds)
+	require.Equal(t, []string{"edge", "beta"}, ctx.Config.Snapcrafts[0].ChannelTemplates)
+	require.Equal(t, "devel", ctx.Config.Snapcrafts[0].Grade)
+}
+
+func TestDefaultGradeTmplError(t *testing.T) {
+	ctx := testctx.NewWithCfg(config.Project{
+		Builds:     []config.Build{{ID: "foo"}},
+		Snapcrafts: []config.Snapcraft{{Grade: "{{.Env.Grade}}"}},
+	})
+	testlib.RequireTemplateError(t, Pipe{}.Default(ctx))
 }
 
 func TestPublish(t *testing.T) {
-	ctx := context.New(config.Project{})
+	ctx := testctx.New()
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Name:   "mybin",
 		Path:   "nope.snap",
@@ -526,8 +524,7 @@ func TestPublish(t *testing.T) {
 }
 
 func TestPublishSkip(t *testing.T) {
-	ctx := context.New(config.Project{})
-	ctx.SkipPublish = true
+	ctx := testctx.New(testctx.SkipPublish)
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Name:   "mybin",
 		Path:   "nope.snap",
@@ -542,7 +539,7 @@ func TestPublishSkip(t *testing.T) {
 }
 
 func TestDefaultSet(t *testing.T) {
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		Snapcrafts: []config.Snapcraft{
 			{
 				ID:           "devel",
@@ -563,8 +560,8 @@ func TestDefaultSet(t *testing.T) {
 }
 
 func Test_processChannelsTemplates(t *testing.T) {
-	ctx := &context.Context{
-		Config: config.Project{
+	ctx := testctx.NewWithCfg(
+		config.Project{
 			Builds: []config.Build{
 				{
 					ID: "default",
@@ -579,26 +576,16 @@ func Test_processChannelsTemplates(t *testing.T) {
 					},
 				},
 			},
+			Env: []string{"FOO=123"},
 		},
-	}
-
-	ctx.SkipPublish = true
-	ctx.Env = map[string]string{
-		"FOO": "123",
-	}
-	ctx.Version = "1.0.0"
-	ctx.Git = context.GitInfo{
-		CurrentTag: "v1.0.0",
-		Commit:     "a1b2c3d4",
-	}
-	ctx.Semver = context.Semver{
-		Major: 1,
-		Minor: 0,
-		Patch: 0,
-	}
+		testctx.SkipPublish,
+		testctx.WithCommit("a1b2c3d4"),
+		testctx.WithCurrentTag("v1.0.0"),
+		testctx.WithSemver(1, 0, 0, ""),
+		testctx.WithVersion("1.0.0"),
+	)
 
 	require.NoError(t, Pipe{}.Default(ctx))
-
 	snap := ctx.Config.Snapcrafts[0]
 	require.Equal(t, "mybin", snap.Name)
 
@@ -662,18 +649,16 @@ func addBinaries(t *testing.T, ctx *context.Context, name, dist string) {
 }
 
 func TestSeveralSnapssWithTheSameID(t *testing.T) {
-	ctx := &context.Context{
-		Config: config.Project{
-			Snapcrafts: []config.Snapcraft{
-				{
-					ID: "a",
-				},
-				{
-					ID: "a",
-				},
+	ctx := testctx.NewWithCfg(config.Project{
+		Snapcrafts: []config.Snapcraft{
+			{
+				ID: "a",
+			},
+			{
+				ID: "a",
 			},
 		},
-	}
+	})
 	require.EqualError(t, Pipe{}.Default(ctx), "found 2 snapcrafts with the ID 'a', please fix your config")
 }
 
@@ -699,11 +684,11 @@ func Test_isValidArch(t *testing.T) {
 
 func TestSkip(t *testing.T) {
 	t.Run("skip", func(t *testing.T) {
-		require.True(t, Pipe{}.Skip(context.New(config.Project{})))
+		require.True(t, Pipe{}.Skip(testctx.New()))
 	})
 
 	t.Run("dont skip", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			Snapcrafts: []config.Snapcraft{
 				{},
 			},
@@ -712,6 +697,15 @@ func TestSkip(t *testing.T) {
 	})
 }
 
+func TestDependencies(t *testing.T) {
+	ctx := testctx.NewWithCfg(config.Project{
+		Snapcrafts: []config.Snapcraft{
+			{},
+		},
+	})
+	require.Equal(t, []string{"snapcraft"}, Pipe{}.Dependencies(ctx))
+}
+
 func requireEqualFileConents(tb testing.TB, a, b string) {
 	tb.Helper()
 	eq, err := gio.EqualFileContents(a, b)
