@@ -2,8 +2,9 @@ package telegram
 
 import (
 	"fmt"
+	"strconv"
 
-	"github.com/caarlos0/env/v6"
+	"github.com/caarlos0/env/v7"
 	"github.com/caarlos0/log"
 	api "github.com/go-telegram-bot-api/telegram-bot-api"
 	"github.com/goreleaser/goreleaser/internal/tmpl"
@@ -29,7 +30,17 @@ func (Pipe) Default(ctx *context.Context) error {
 }
 
 func (Pipe) Announce(ctx *context.Context) error {
-	msg, err := tmpl.New(ctx).Apply(ctx.Config.Announce.Telegram.MessageTemplate)
+	tpl := tmpl.New(ctx)
+	msg, err := tpl.Apply(ctx.Config.Announce.Telegram.MessageTemplate)
+	if err != nil {
+		return fmt.Errorf("telegram: %w", err)
+	}
+
+	chatIDStr, err := tpl.Apply(ctx.Config.Announce.Telegram.ChatID)
+	if err != nil {
+		return fmt.Errorf("telegram: %w", err)
+	}
+	chatID, err := strconv.ParseInt(chatIDStr, 10, 64)
 	if err != nil {
 		return fmt.Errorf("telegram: %w", err)
 	}
@@ -45,7 +56,7 @@ func (Pipe) Announce(ctx *context.Context) error {
 		return fmt.Errorf("telegram: %w", err)
 	}
 
-	tm := api.NewMessage(ctx.Config.Announce.Telegram.ChatID, msg)
+	tm := api.NewMessage(chatID, msg)
 	tm.ParseMode = "MarkdownV2"
 	_, err = bot.Send(tm)
 	if err != nil {
