@@ -12,6 +12,8 @@ import (
 
 	"github.com/goreleaser/goreleaser/internal/artifact"
 	"github.com/goreleaser/goreleaser/internal/pipe"
+	"github.com/goreleaser/goreleaser/internal/testctx"
+	"github.com/goreleaser/goreleaser/internal/testlib"
 	"github.com/goreleaser/goreleaser/pkg/config"
 	"github.com/goreleaser/goreleaser/pkg/context"
 	"github.com/stretchr/testify/require"
@@ -99,7 +101,7 @@ func TestRunPipe_ModeBinary(t *testing.T) {
 		w.WriteHeader(http.StatusCreated)
 	})
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Uploads: []config.Upload{
@@ -118,14 +120,12 @@ func TestRunPipe_ModeBinary(t *testing.T) {
 				Username: "productionuser",
 			},
 		},
-		Archives: []config.Archive{
-			{},
+		Archives: []config.Archive{{}},
+		Env: []string{
+			"UPLOAD_PRODUCTION-US_SECRET=deployuser-secret",
+			"UPLOAD_PRODUCTION-EU_SECRET=productionuser-apikey",
 		},
 	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION-US_SECRET": "deployuser-secret",
-		"UPLOAD_PRODUCTION-EU_SECRET": "productionuser-apikey",
-	}
 	for _, goos := range []string{"linux", "darwin"} {
 		ctx.Artifacts.Add(&artifact.Artifact{
 			Name:   "mybin",
@@ -151,7 +151,7 @@ func TestRunPipe_ModeArchive(t *testing.T) {
 	require.NoError(t, err)
 	require.NoError(t, debfile.Close())
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "goreleaser",
 		Dist:        folder,
 		Uploads: []config.Upload{
@@ -163,14 +163,9 @@ func TestRunPipe_ModeArchive(t *testing.T) {
 				Username: "deployuser",
 			},
 		},
-		Archives: []config.Archive{
-			{},
-		},
-	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
-	ctx.Version = "1.0.0"
+		Archives: []config.Archive{{}},
+		Env:      []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
+	}, testctx.WithVersion("1.0.0"))
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Type: artifact.UploadableArchive,
 		Name: "bin.tar.gz",
@@ -243,7 +238,7 @@ func TestRunPipe_ModeBinary_CustomArtifactName(t *testing.T) {
 		w.WriteHeader(http.StatusCreated)
 	})
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Uploads: []config.Upload{
@@ -256,13 +251,9 @@ func TestRunPipe_ModeBinary_CustomArtifactName(t *testing.T) {
 				CustomArtifactName: true,
 			},
 		},
-		Archives: []config.Archive{
-			{},
-		},
+		Archives: []config.Archive{{}},
+		Env:      []string{"UPLOAD_PRODUCTION-US_SECRET=deployuser-secret"},
 	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION-US_SECRET": "deployuser-secret",
-	}
 	for _, goos := range []string{"linux", "darwin"} {
 		ctx.Artifacts.Add(&artifact.Artifact{
 			Name:   "mybin",
@@ -288,7 +279,7 @@ func TestRunPipe_ModeArchive_CustomArtifactName(t *testing.T) {
 	require.NoError(t, err)
 	require.NoError(t, debfile.Close())
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "goreleaser",
 		Dist:        folder,
 		Uploads: []config.Upload{
@@ -301,14 +292,11 @@ func TestRunPipe_ModeArchive_CustomArtifactName(t *testing.T) {
 				CustomArtifactName: true,
 			},
 		},
-		Archives: []config.Archive{
-			{},
+		Archives: []config.Archive{{}},
+		Env: []string{
+			"UPLOAD_PRODUCTION_SECRET=deployuser-secret",
 		},
-	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
-	ctx.Version = "1.0.0"
+	}, testctx.WithVersion("1.0.0"))
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Type: artifact.UploadableArchive,
 		Name: "bin.tar.gz",
@@ -355,7 +343,7 @@ func TestRunPipe_ArtifactoryDown(t *testing.T) {
 	require.NoError(t, err)
 	require.NoError(t, tarfile.Close())
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "goreleaser",
 		Dist:        folder,
 		Uploads: []config.Upload{
@@ -367,11 +355,8 @@ func TestRunPipe_ArtifactoryDown(t *testing.T) {
 				Username: "deployuser",
 			},
 		},
-	})
-	ctx.Version = "2.0.0"
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
+		Env: []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
+	}, testctx.WithVersion("2.0.0"))
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Type: artifact.UploadableArchive,
 		Name: "bin.tar.gz",
@@ -387,7 +372,7 @@ func TestRunPipe_TargetTemplateError(t *testing.T) {
 	dist := filepath.Join(folder, "dist")
 	binPath := filepath.Join(dist, "mybin", "mybin")
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Uploads: []config.Upload{
@@ -396,17 +381,13 @@ func TestRunPipe_TargetTemplateError(t *testing.T) {
 				Name:   "production",
 				Mode:   "binary",
 				// This template is not correct and should fail
-				Target:   "http://storage.company.com/example-repo-local/{{ .ProjectName /{{ .Version }}/",
+				Target:   "http://storage.company.com/example-repo-local/{{ .ProjectName}",
 				Username: "deployuser",
 			},
 		},
-		Archives: []config.Archive{
-			{},
-		},
+		Archives: []config.Archive{{}},
+		Env:      []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
 	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Name:   "mybin",
 		Path:   binPath,
@@ -414,9 +395,7 @@ func TestRunPipe_TargetTemplateError(t *testing.T) {
 		Goos:   "darwin",
 		Type:   artifact.UploadableBinary,
 	})
-	err := Pipe{}.Publish(ctx)
-	require.Error(t, err)
-	require.Contains(t, err.Error(), `upload: error while building the target url: template: tmpl:1: unexpected "/" in operand`)
+	testlib.RequireTemplateError(t, Pipe{}.Publish(ctx))
 }
 
 func TestRunPipe_BadCredentials(t *testing.T) {
@@ -441,7 +420,7 @@ func TestRunPipe_BadCredentials(t *testing.T) {
 		w.WriteHeader(http.StatusUnauthorized)
 	})
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Uploads: []config.Upload{
@@ -453,13 +432,9 @@ func TestRunPipe_BadCredentials(t *testing.T) {
 				Username: "deployuser",
 			},
 		},
-		Archives: []config.Archive{
-			{},
-		},
+		Archives: []config.Archive{{}},
+		Env:      []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
 	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Name:   "mybin",
 		Path:   binPath,
@@ -486,13 +461,9 @@ func TestRunPipe_FileNotFound(t *testing.T) {
 				Username: "deployuser",
 			},
 		},
-		Archives: []config.Archive{
-			{},
-		},
+		Archives: []config.Archive{{}},
+		Env:      []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
 	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Name:   "mybin",
 		Path:   "archivetest/dist/mybin/mybin",
@@ -501,7 +472,7 @@ func TestRunPipe_FileNotFound(t *testing.T) {
 		Type:   artifact.UploadableBinary,
 	})
 
-	require.EqualError(t, Pipe{}.Publish(ctx), `open archivetest/dist/mybin/mybin: no such file or directory`)
+	require.ErrorIs(t, Pipe{}.Publish(ctx), os.ErrNotExist)
 }
 
 func TestRunPipe_UnparsableTarget(t *testing.T) {
@@ -513,7 +484,7 @@ func TestRunPipe_UnparsableTarget(t *testing.T) {
 	d1 := []byte("hello\ngo\n")
 	require.NoError(t, os.WriteFile(binPath, d1, 0o666))
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Uploads: []config.Upload{
@@ -528,6 +499,7 @@ func TestRunPipe_UnparsableTarget(t *testing.T) {
 		Archives: []config.Archive{
 			{},
 		},
+		Env: []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
 	})
 	ctx.Env = map[string]string{
 		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
@@ -550,7 +522,7 @@ func TestRunPipe_DirUpload(t *testing.T) {
 	require.NoError(t, os.Mkdir(filepath.Join(dist, "mybin"), 0o755))
 	binPath := filepath.Join(dist, "mybin")
 
-	ctx := context.New(config.Project{
+	ctx := testctx.NewWithCfg(config.Project{
 		ProjectName: "mybin",
 		Dist:        dist,
 		Uploads: []config.Upload{
@@ -562,13 +534,9 @@ func TestRunPipe_DirUpload(t *testing.T) {
 				Username: "deployuser",
 			},
 		},
-		Archives: []config.Archive{
-			{},
-		},
+		Archives: []config.Archive{{}},
+		Env:      []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
 	})
-	ctx.Env = map[string]string{
-		"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-	}
 	ctx.Artifacts.Add(&artifact.Artifact{
 		Name:   "mybin",
 		Path:   filepath.Dir(binPath),
@@ -585,45 +553,37 @@ func TestDescription(t *testing.T) {
 }
 
 func TestPutsWithoutTarget(t *testing.T) {
-	ctx := &context.Context{
-		Env: map[string]string{
-			"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-		},
-		Config: config.Project{
-			Uploads: []config.Upload{
-				{
-					Method:   h.MethodPut,
-					Name:     "production",
-					Username: "deployuser",
-				},
+	ctx := testctx.NewWithCfg(config.Project{
+		Uploads: []config.Upload{
+			{
+				Method:   h.MethodPut,
+				Name:     "production",
+				Username: "deployuser",
 			},
 		},
-	}
+		Env: []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
+	})
 
 	require.True(t, pipe.IsSkip(Pipe{}.Publish(ctx)))
 }
 
 func TestPutsWithoutUsername(t *testing.T) {
-	ctx := &context.Context{
-		Env: map[string]string{
-			"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-		},
-		Config: config.Project{
-			Uploads: []config.Upload{
-				{
-					Method: h.MethodPut,
-					Name:   "production",
-					Target: "http://artifacts.company.com/example-repo-local/{{ .ProjectName }}/{{ .Os }}/{{ .Arch }}{{ if .Arm }}v{{ .Arm }}{{ end }}",
-				},
+	ctx := testctx.NewWithCfg(config.Project{
+		Uploads: []config.Upload{
+			{
+				Method: h.MethodPut,
+				Name:   "production",
+				Target: "http://artifacts.company.com/example-repo-local/{{ .ProjectName }}/{{ .Os }}/{{ .Arch }}{{ if .Arm }}v{{ .Arm }}{{ end }}",
 			},
 		},
-	}
+		Env: []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
+	})
 
 	require.True(t, pipe.IsSkip(Pipe{}.Publish(ctx)))
 }
 
 func TestPutsWithoutName(t *testing.T) {
-	require.True(t, pipe.IsSkip(Pipe{}.Publish(context.New(config.Project{
+	require.True(t, pipe.IsSkip(Pipe{}.Publish(testctx.NewWithCfg(config.Project{
 		Uploads: []config.Upload{
 			{
 				Method:   h.MethodPut,
@@ -635,7 +595,7 @@ func TestPutsWithoutName(t *testing.T) {
 }
 
 func TestPutsWithoutSecret(t *testing.T) {
-	require.True(t, pipe.IsSkip(Pipe{}.Publish(context.New(config.Project{
+	require.True(t, pipe.IsSkip(Pipe{}.Publish(testctx.NewWithCfg(config.Project{
 		Uploads: []config.Upload{
 			{
 				Method:   h.MethodPut,
@@ -648,37 +608,31 @@ func TestPutsWithoutSecret(t *testing.T) {
 }
 
 func TestPutsWithInvalidMode(t *testing.T) {
-	ctx := &context.Context{
-		Env: map[string]string{
-			"UPLOAD_PRODUCTION_SECRET": "deployuser-secret",
-		},
-		Config: config.Project{
-			Uploads: []config.Upload{
-				{
-					Method:   h.MethodPut,
-					Name:     "production",
-					Mode:     "does-not-exists",
-					Target:   "http://artifacts.company.com/example-repo-local/{{ .ProjectName }}/{{ .Os }}/{{ .Arch }}{{ if .Arm }}v{{ .Arm }}{{ end }}",
-					Username: "deployuser",
-				},
+	ctx := testctx.NewWithCfg(config.Project{
+		Uploads: []config.Upload{
+			{
+				Method:   h.MethodPut,
+				Name:     "production",
+				Mode:     "does-not-exists",
+				Target:   "http://artifacts.company.com/example-repo-local/{{ .ProjectName }}/{{ .Os }}/{{ .Arch }}{{ if .Arm }}v{{ .Arm }}{{ end }}",
+				Username: "deployuser",
 			},
 		},
-	}
+		Env: []string{"UPLOAD_PRODUCTION_SECRET=deployuser-secret"},
+	})
 	require.Error(t, Pipe{}.Publish(ctx))
 }
 
 func TestDefault(t *testing.T) {
-	ctx := &context.Context{
-		Config: config.Project{
-			Uploads: []config.Upload{
-				{
-					Name:     "production",
-					Target:   "http://artifacts.company.com/example-repo-local/{{ .ProjectName }}/{{ .Os }}/{{ .Arch }}{{ if .Arm }}v{{ .Arm }}{{ end }}",
-					Username: "deployuser",
-				},
+	ctx := testctx.NewWithCfg(config.Project{
+		Uploads: []config.Upload{
+			{
+				Name:     "production",
+				Target:   "http://artifacts.company.com/example-repo-local/{{ .ProjectName }}/{{ .Os }}/{{ .Arch }}{{ if .Arm }}v{{ .Arm }}{{ end }}",
+				Username: "deployuser",
 			},
 		},
-	}
+	})
 	require.NoError(t, Pipe{}.Default(ctx))
 	require.Len(t, ctx.Config.Uploads, 1)
 	upload := ctx.Config.Uploads[0]
@@ -687,26 +641,22 @@ func TestDefault(t *testing.T) {
 }
 
 func TestDefaultNoPuts(t *testing.T) {
-	ctx := &context.Context{
-		Config: config.Project{
-			Uploads: []config.Upload{},
-		},
-	}
+	ctx := testctx.NewWithCfg(config.Project{
+		Uploads: []config.Upload{},
+	})
 	require.NoError(t, Pipe{}.Default(ctx))
 	require.Empty(t, ctx.Config.Uploads)
 }
 
 func TestDefaultSet(t *testing.T) {
-	ctx := &context.Context{
-		Config: config.Project{
-			Uploads: []config.Upload{
-				{
-					Method: h.MethodPost,
-					Mode:   "custom",
-				},
+	ctx := testctx.NewWithCfg(config.Project{
+		Uploads: []config.Upload{
+			{
+				Method: h.MethodPost,
+				Mode:   "custom",
 			},
 		},
-	}
+	})
 	require.NoError(t, Pipe{}.Default(ctx))
 	require.Len(t, ctx.Config.Uploads, 1)
 	upload := ctx.Config.Uploads[0]
@@ -716,11 +666,11 @@ func TestDefaultSet(t *testing.T) {
 
 func TestSkip(t *testing.T) {
 	t.Run("skip", func(t *testing.T) {
-		require.True(t, Pipe{}.Skip(context.New(config.Project{})))
+		require.True(t, Pipe{}.Skip(testctx.New()))
 	})
 
 	t.Run("dont skip", func(t *testing.T) {
-		ctx := context.New(config.Project{
+		ctx := testctx.NewWithCfg(config.Project{
 			Uploads: []config.Upload{
 				{},
 			},
