@@ -78,7 +78,7 @@ type Context struct {
 	TokenType          TokenType
 	Git                GitInfo
 	Date               time.Time
-	Artifacts          artifact.Artifacts
+	Artifacts          *artifact.Artifacts
 	ReleaseURL         string
 	ReleaseNotes       string
 	ReleaseNotesFile   string
@@ -96,9 +96,10 @@ type Context struct {
 	SkipSign           bool
 	SkipValidate       bool
 	SkipSBOMCataloging bool
+	SkipKo             bool
 	SkipDocker         bool
 	SkipBefore         bool
-	RmDist             bool
+	Clean              bool
 	PreRelease         bool
 	Deprecated         bool
 	Parallelism        int
@@ -116,7 +117,6 @@ type Semver struct {
 	Major      uint64
 	Minor      uint64
 	Patch      uint64
-	RawVersion string
 	Prerelease string
 }
 
